/***************************************************************************
                          kplayer.cpp
                          -----------
    begin                : Sat Nov 16 10:12:50 EST 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#define _ISOC99_SOURCE
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <kurl.h>
#include <kurldrag.h>
#include <math.h>
#include <qcursor.h>
#include <qdockarea.h>
#include <qlayout.h>
#include <qobjectlist.h>
#include <dcopclient.h>

#include <kdebug.h>

#include "kplayer.h"
#include "kplayerlogwindow.h"
#include "kplayersettings.h"
#include "kplayerslideraction.h"

#define ID_STATUS_MSG   1
#define ID_STATE_MSG    2
#define ID_PROGRESS_MSG 3

KPlayerApplication::KPlayerApplication (void)
  : KUniqueApplication (true, true, true),
    m_settings (0)
{
}

KPlayerApplication::~KPlayerApplication()
{
  if ( m_settings )
    delete m_settings;
}

int KPlayerApplication::newInstance (void)
{
  kdDebug() << "New instance\n";
  if ( ! m_settings )
    m_settings = new KPlayerSettings;
  KPlayer* mainWindow = (KPlayer*) mainWidget();
  if ( ! mainWindow )
  {
    kdDebug() << "Creating main window\n";
    mainWindow = new KPlayer();
    setMainWidget (mainWindow);
  }
  mainWindow -> start();
  return 0;
}

KPlayer::KPlayer (QWidget *parent, const char *name) : KMainWindow (parent, name)
{
  m_resizing = true;
  m_updating = m_initial_fullscreen = m_enable_screen_saver = false;
  m_status_label = m_state_label = m_progress_label = 0;
  m_menubar_normally_visible = m_statusbar_normally_visible = true;
  m_menubar_fullscreen_visible = m_statusbar_fullscreen_visible = false;
  m_messagelog_normally_visible = m_messagelog_fullscreen_visible = false;
  Toolbar toolbar [KPLAYER_TOOLBARS] = {
    { "mainToolBar", KStdAction::stdName (KStdAction::ShowToolbar), true, false },
    { "progressToolBar", "settings_progress_toolbar", true, false },
    { "volumeToolBar", "settings_volume_toolbar", false, false },
    { "contrastToolBar", "settings_contrast_toolbar", false, false },
    { "brightnessToolBar", "settings_brightness_toolbar", false, false },
    { "hueToolBar", "settings_hue_toolbar", false, false },
    { "saturationToolBar", "settings_saturation_toolbar", false, false }
  };
  QString captions [KPLAYER_TOOLBARS] = {
    i18n("Main Toolbar"),
    i18n("Progress"),
    i18n("Volume"),
    i18n("Contrast"),
    i18n("Brightness"),
    i18n("Hue"),
    i18n("Saturation")
  };
  QString actions [KPLAYER_TOOLBARS - 1] = {
    i18n("player_progress"),
    i18n("player_volume"),
    i18n("video_contrast"),
    i18n("video_brightness"),
    i18n("video_hue"),
    i18n("video_saturation")
  };
  KPlayerWorkspace* workspace = new KPlayerWorkspace (this);
  connect (actionCollection(), SIGNAL (actionStatusText(const QString&)), this, SLOT (setStatusMessage(const QString&)));
  connect (actionCollection(), SIGNAL (clearStatusText()), this, SLOT (clearStatusMessage()));
  actionCollection() -> setHighlightingEnabled (true);
  connect (workspace -> kPlayerWidget(), SIGNAL (resized()), this, SLOT (widgetResized()));
  connect (workspace -> kPlayerProcess(), SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (playerStateChanged(KPlayerProcess::State)));
  connect (workspace -> kPlayerProcess(), SIGNAL (progressChanged(float)), this, SLOT (playerProgressChanged(float)));
  connect (workspace -> kPlayerProcess(), SIGNAL (infoAvailable()), this, SLOT (playerInfoAvailable()));
  connect (workspace -> kPlayerProcess(), SIGNAL (sizeAvailable()), this, SLOT (playerSizeAvailable()));
  connect (workspace -> kPlayerProcess(), SIGNAL (messageReceived(const char*)), this, SLOT (playerMessageReceived(const char*)));
  connect (workspace -> kPlayerProcess(), SIGNAL (errorDetected()), this, SLOT (playerErrorDetected()));
  connect (kPlayerSettings(), SIGNAL (saved()), this, SLOT (refreshSettings()));
  setCentralWidget (workspace);
  setAutoSaveSettings ("MainWindow", false);
  resetAutoSaveSettings();  // saveWindowSize = false;
  m_log = new KPlayerLogWindow (this);
  connect (m_log, SIGNAL (windowHidden()), this, SLOT (logWindowHidden()));
  initStatusBar();
  initActions();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    m_toolbar[i] = toolbar[i];
/*  connect (toolBar (m_toolbar[i].name), SIGNAL (orientationChanged(Orientation)), this, SLOT (barOrientationChanged(Orientation)));
    connect (toolBar (m_toolbar[i].name), SIGNAL (placeChanged(QDockWindow::Place)), this, SLOT (barPlaceChanged(QDockWindow::Place)));
    connect (toolBar (m_toolbar[i].name), SIGNAL (visibilityChanged(bool)), this, SLOT (barVisibilityChanged(bool)));
  }*/
  readOptions();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
  {
    KToolBar* toolbar = toolBar (m_toolbar[i].name);
    toolbar -> setCaption (captions[i]);
    //kdDebug() << "Slider " << long (sliderAction (m_toolbar[i].action) -> slider()) << "\n";
    //kdDebug() << "Orientation " << sliderAction (m_toolbar[i].action) -> slider() -> orientation() << "\n";
    if ( i )
      sliderAction (actions [i - 1]) -> slider() -> setOrientation (toolbar -> orientation());
    /*if ( toolbar -> orientation() != Qt::Horizontal )
    {
      // force orientationChanged event
      toolbar -> undock();
      toolbar -> dock();
      if ( ! toggleAction (m_toolbar[i].action) -> isChecked() )
        toolbar -> hide();
    }*/
  }
  enablePlayerActions();
  enableVideoActions();
  setEraseColor (QColor (0, 0, 0));
  setAcceptDrops (true);
  m_resizing = false;
}

void KPlayer::initActions (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  KAction* action = KStdAction::open (this, SLOT (fileOpen()), actionCollection());
  action -> setStatusText (i18n("Opens an existing file"));
  action = new KAction (i18n("Open &URL..."), "fileopenurl", CTRL + Key_U, this, SLOT (fileOpenUrl()), actionCollection(), "file_open_url");
  action -> setStatusText (i18n("Opens a URL"));
  KRecentFilesAction* recent = KStdAction::openRecent (this, SLOT (fileOpenRecent(const KURL&)), actionCollection());
  recent -> setStatusText (i18n("Opens a recently used file"));
  recent -> setMaxItems (settings -> recentFileListSize());
//recent -> setEnabled (false);
  action = new KAction (i18n("&Load Subtitles..."), 0, CTRL + Key_S, this, SLOT (fileOpenSubtitles()), actionCollection(), "subtitles_load");
  action -> setStatusText (i18n("Opens a subtitle file"));
  action = new KAction (i18n("Load Su&btitle URL..."), 0, 0, this, SLOT (fileOpenSubtitleUrl()), actionCollection(), "subtitles_load_url");
  action -> setStatusText (i18n("Opens a subtitle URL"));
  action = new KAction (i18n("U&nload Subtitles"), 0, 0, this, SLOT (fileUnloadSubtitles()), actionCollection(), "subtitles_unload");
  action -> setStatusText (i18n("Unloads subtitles"));
  action = new KAction (i18n("&Clear Log"), 0, 0, this, SLOT (fileClearLog()), actionCollection(), "file_clear_log");
  action -> setStatusText (i18n("Clears all messages from the message log"));
  action = KStdAction::quit (this, SLOT (fileQuit()), actionCollection());
  action -> setStatusText (i18n("Quits KPlayer"));

  action = new KToggleAction (i18n("&Full Screen"), "window_fullscreen", CTRL + Key_F, this, SLOT (viewFullScreen()), actionCollection(), "view_full_screen");
  action -> setStatusText (i18n("Switches to full screen mode"));
  action = KStdAction::zoomIn (this, SLOT (viewZoomIn()), actionCollection());
  action -> setStatusText (i18n("Enlarges the video"));
  action = KStdAction::zoomOut (this, SLOT (viewZoomOut()), actionCollection());
  action -> setStatusText (i18n("Shrinks the video"));
  KToggleAction* toggle = new KToggleAction (i18n("Zoom to 5&0%"), 0, 0, this, SLOT (viewZoom12()), actionCollection(), "view_zoom_1_2");
  toggle -> setStatusText (i18n("Scales video to 50% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &100%"), 0, CTRL + Key_Backspace, this, SLOT (viewZoom11()), actionCollection(), "view_zoom_1_1");
  toggle -> setStatusText (i18n("Scales video to its original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoo&m to 150%"), 0, 0, this, SLOT (viewZoom32()), actionCollection(), "view_zoom_3_2");
  toggle -> setStatusText (i18n("Scales video to 150% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &200%"), 0, 0, this, SLOT (viewZoom21()), actionCollection(), "view_zoom_2_1");
  toggle -> setStatusText (i18n("Scales video to 200% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom &to 250%"), 0, 0, this, SLOT (viewZoom52()), actionCollection(), "view_zoom_5_2");
  toggle -> setStatusText (i18n("Scales video to 250% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &300%"), 0, 0, this, SLOT (viewZoom31()), actionCollection(), "view_zoom_3_1");
  toggle -> setStatusText (i18n("Scales video to 300% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Maintain Original &Aspect"), "viewmagfit", CTRL + Key_A, this, SLOT (viewMaintainOriginalAspect()), actionCollection(), "view_original_aspect");
  toggle -> setStatusText (i18n("Maintains the original video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Maintain &Current Aspect"), 0, 0, this, SLOT (viewMaintainCurrentAspect()), actionCollection(), "view_current_aspect");
  toggle -> setStatusText (i18n("Maintains the current video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Force &4 to 3 Aspect"), 0, 0, this, SLOT (viewAspect43()), actionCollection(), "view_aspect_4_3");
  toggle -> setStatusText (i18n("Sets 4 to 3 video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Force 1&6 to 9 Aspect"), 0, 0, this, SLOT (viewAspect169()), actionCollection(), "view_aspect_16_9");
  toggle -> setStatusText (i18n("Sets 16 to 9 video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");

  action = new KAction (i18n("&Play"), "player_play", Key_Return, this, SLOT (playerPlay()), actionCollection(), "player_play");
  action -> setStatusText (i18n("Plays the currently loaded file"));
  action = new KToggleAction (i18n("Paus&e"), "player_pause", Key_Space, this, SLOT (playerPause()), actionCollection(), "player_pause");
  action -> setStatusText (i18n("Pauses the player"));
  action = new KAction (i18n("&Stop"), "player_stop", Key_Escape, this, SLOT (playerStop()), actionCollection(), "player_stop");
  action -> setStatusText (i18n("Stops the player"));

  action = new KAction (i18n("&Forward"), 0, Key_Right, this, SLOT (playerForward()), actionCollection(), "player_forward");
  action -> setStatusText (i18n("Moves forward by one percent of video length"));
  action = new KAction (i18n("F&ast Forward"), "player_fwd", CTRL + Key_Right, this, SLOT (playerFastForward()), actionCollection(), "player_fast_forward");
  action -> setStatusText (i18n("Moves forward by ten percent of video length"));
  action = new KAction (i18n("&Backward"), 0, Key_Left, this, SLOT (playerBackward()), actionCollection(), "player_backward");
  action -> setStatusText (i18n("Moves backward by one percent of video length"));
  action = new KAction (i18n("Fast Back&ward"), "player_rew", CTRL + Key_Left, this, SLOT (playerFastBackward()), actionCollection(), "player_fast_backward");
  action -> setStatusText (i18n("Moves backward by ten percent of video length"));
  action = new KAction (i18n("Back &to Start"), 0, CTRL + Key_Home, this, SLOT (playerStart()), actionCollection(), "player_start");
  action -> setStatusText (i18n("Moves to the beginning"));

  action = new KAction (i18n("Volume &Up"), 0, Key_Up, this, SLOT (playerVolumeUp()), actionCollection(), "player_volume_up");
  action -> setStatusText (i18n("Increases the volume"));
  action = new KAction (i18n("Volume &Down"), 0, Key_Down, this, SLOT (playerVolumeDown()), actionCollection(), "player_volume_down");
  action -> setStatusText (i18n("Decreases the volume"));
  action = new KToggleAction (i18n("&Mute"), "mute", CTRL + Key_Backslash, this, SLOT (playerMute()), actionCollection(), "player_mute");
  action -> setStatusText (i18n("Turns the sound on/off"));

  toggle = new KToggleAction (i18n("S&oft Frame Dropping"), 0, 0, this, SLOT (playerSoftFrameDrop()), actionCollection(), "player_soft_frame_drop");
  toggle -> setStatusText (i18n("Turns soft frame dropping on/off"));
  toggle -> setExclusiveGroup ("framedrop");
  toggle = new KToggleAction (i18n("&Hard Frame Dropping"), 0, 0, this, SLOT (playerHardFrameDrop()), actionCollection(), "player_hard_frame_drop");
  toggle -> setStatusText (i18n("Turns hard frame dropping on/off"));
  toggle -> setExclusiveGroup ("framedrop");

  toggle = new KToggleAction (i18n("&Show"), 0, 0, this, SLOT (playerSubtitlesShow()), actionCollection(), "subtitles_show");
  toggle -> setStatusText (i18n("Turns subtitle display on/off"));
  action = new KAction (i18n("Move &Down"), 0, CTRL + Key_Comma, this, SLOT (playerSubtitlesMoveDown()), actionCollection(), "subtitles_move_down");
  action -> setStatusText (i18n("Moves subtitles down"));
  action = new KAction (i18n("Move &Up"), 0, CTRL + Key_Period, this, SLOT (playerSubtitlesMoveUp()), actionCollection(), "subtitles_move_up");
  action -> setStatusText (i18n("Moves subtitles up"));
  action = new KAction (i18n("D&ecrease Delay"), 0, CTRL + Key_Less, this, SLOT (playerSubtitlesDelayDecrease()), actionCollection(), "subtitles_delay_decrease");
  action -> setStatusText (i18n("Decreases subtitle delay"));
  action = new KAction (i18n("&Increase Delay"), 0, CTRL + Key_Greater, this, SLOT (playerSubtitlesDelayIncrease()), actionCollection(), "subtitles_delay_increase");
  action -> setStatusText (i18n("Increases subtitle delay"));

  action = new KAction (i18n("C&ontrast Down"), 0, CTRL + Key_1, this, SLOT (videoContrastDown()), actionCollection(), "video_contrast_down");
  action -> setStatusText (i18n("Decreases the video contrast"));
  action = new KAction (i18n("&Contrast Up"), 0, CTRL + Key_2, this, SLOT (videoContrastUp()), actionCollection(), "video_contrast_up");
  action -> setStatusText (i18n("Increases the video contrast"));
  action = new KAction (i18n("B&rightness Down"), 0, CTRL + Key_3, this, SLOT (videoBrightnessDown()), actionCollection(), "video_brightness_down");
  action -> setStatusText (i18n("Decreases the video brightness"));
  action = new KAction (i18n("&Brightness Up"), 0, CTRL + Key_4, this, SLOT (videoBrightnessUp()), actionCollection(), "video_brightness_up");
  action -> setStatusText (i18n("Increases the video brightness"));
  action = new KAction (i18n("H&ue Down"), 0, CTRL + Key_5, this, SLOT (videoHueDown()), actionCollection(), "video_hue_down");
  action -> setStatusText (i18n("Decreases the video hue"));
  action = new KAction (i18n("&Hue Up"), 0, CTRL + Key_6, this, SLOT (videoHueUp()), actionCollection(), "video_hue_up");
  action -> setStatusText (i18n("Increases the video hue"));
  action = new KAction (i18n("S&aturation Down"), 0, CTRL + Key_7, this, SLOT (videoSaturationDown()), actionCollection(), "video_saturation_down");
  action -> setStatusText (i18n("Decreases the video saturation"));
  action = new KAction (i18n("&Saturation Up"), 0, CTRL + Key_8, this, SLOT (videoSaturationUp()), actionCollection(), "video_saturation_up");
  action -> setStatusText (i18n("Increases the video saturation"));

  action = KStdAction::showMenubar (this, SLOT (viewMenubar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the menu bar"));
  action = KStdAction::showStatusbar (this, SLOT (viewStatusbar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the status bar"));
  action = new KToggleAction (i18n("Show Message &Log"), 0, CTRL + Key_L, this, SLOT (viewMessageLog()), actionCollection(), "options_show_log");
  action -> setStatusText (i18n("Shows/hides the message log"));
  action = KStdAction::showToolbar (this, SLOT (viewMainToolbar()), actionCollection());
  action -> setText (i18n("Show Main &Toolbar"));
  action -> setStatusText (i18n("Shows/hides the main toolbar"));
  action = new KToggleAction (i18n("Show &Progress Toolbar"), 0, 0, this, SLOT (viewProgressToolbar()), actionCollection(), "settings_progress_toolbar");
  action -> setStatusText (i18n("Shows/hides the progress toolbar"));
  action = new KToggleAction (i18n("Show &Volume Toolbar"), "volume", 0, this, SLOT (viewVolumeToolbar()), actionCollection(), "settings_volume_toolbar");
  action -> setStatusText (i18n("Shows/hides the volume toolbar"));
  action = new KToggleAction (i18n("Show C&ontrast Toolbar"), "contrast", 0, this, SLOT (viewContrastToolbar()), actionCollection(), "settings_contrast_toolbar");
  action -> setStatusText (i18n("Shows/hides the contrast toolbar"));
  action = new KToggleAction (i18n("Show &Brightness Toolbar"), "brightness", 0, this, SLOT (viewBrightnessToolbar()), actionCollection(), "settings_brightness_toolbar");
  action -> setStatusText (i18n("Shows/hides the brightness toolbar"));
  action = new KToggleAction (i18n("Show H&ue Toolbar"), "hue", 0, this, SLOT (viewHueToolbar()), actionCollection(), "settings_hue_toolbar");
  action -> setStatusText (i18n("Shows/hides the hue toolbar"));
  action = new KToggleAction (i18n("Show &Saturation Toolbar"), "saturation", 0, this, SLOT (viewSaturationToolbar()), actionCollection(), "settings_saturation_toolbar");
  action -> setStatusText (i18n("Shows/hides the saturation toolbar"));
  action = KStdAction::keyBindings (this, SLOT (settingsKeyBindings()), actionCollection());
  action -> setStatusText (i18n("Opens the KPlayer configuration dialog"));
  action = KStdAction::preferences (this, SLOT (settingsConfigure()), actionCollection());
  action -> setStatusText (i18n("Opens the KPlayer configuration dialog"));

  m_updating = true;
  KPlayerSliderAction* sa = new KPlayerSliderAction (i18n("Progress"), this, SLOT (progressChanged (int)), actionCollection(), "player_progress");
  sa -> slider() -> setup (0, 0, 0, 0, 0);
//sa -> slider() -> setTracking (false);
  sa -> setStatusText (i18n("Shows player progress and allows seeking"));
  sa = new KPlayerSliderAction (i18n("Volume"), this, SLOT (volumeChanged (int)), actionCollection(), "player_volume");
  sa -> setStatusText (i18n("Changes volume level"));
  sa = new KPlayerSliderAction (i18n("Contrast"), this, SLOT (contrastChanged (int)), actionCollection(), "video_contrast");
  sa -> setStatusText (i18n("Changes contrast level"));
  sa = new KPlayerSliderAction (i18n("Brightness"), this, SLOT (brightnessChanged (int)), actionCollection(), "video_brightness");
  sa -> setStatusText (i18n("Changes brightness level"));
  sa = new KPlayerSliderAction (i18n("Hue"), this, SLOT (hueChanged (int)), actionCollection(), "video_hue");
  sa -> setStatusText (i18n("Changes hue level"));
  sa = new KPlayerSliderAction (i18n("Saturation"), this, SLOT (saturationChanged (int)), actionCollection(), "video_saturation");
  sa -> setStatusText (i18n("Changes saturation level"));

  KPlayerPopupSliderAction* psa = new KPlayerPopupSliderAction (i18n("&Volume Popup"), "volume", Key_F9, this, SLOT (volumeChanged (int)), actionCollection(), "popup_volume");
  psa -> setStatusText (i18n("Shows the volume popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Contrast Popup"), "contrast", Key_F5, this, SLOT (contrastChanged (int)), actionCollection(), "popup_contrast");
  psa -> setStatusText (i18n("Shows the contrast popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Brightness Popup"), "brightness", Key_F6, this, SLOT (brightnessChanged (int)), actionCollection(), "popup_brightness");
  psa -> setStatusText (i18n("Shows the brightness popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Hue Popup"), "hue", Key_F7, this, SLOT (hueChanged (int)), actionCollection(), "popup_hue");
  psa -> setStatusText (i18n("Shows the hue popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Saturation Popup"), "saturation", Key_F8, this, SLOT (saturationChanged (int)), actionCollection(), "popup_saturation");
  psa -> setStatusText (i18n("Shows the saturation popup slider"));
  m_updating = false;

  createGUI();

  action = actionCollection() -> action ("help_contents");
  if ( action )
    action -> setEnabled (false);
  action = actionCollection() -> action ("help_whats_this");
  if ( action )
    action -> setEnabled (false);
}

void KPlayer::initStatusBar (void)
{
  statusBar() -> insertItem (i18n("Ready"), ID_STATUS_MSG, 1);
  statusBar() -> insertItem (i18n("Idle"), ID_STATE_MSG, 1, true);
  statusBar() -> insertItem ("", ID_PROGRESS_MSG, 1, true);
  QObjectList* children = (QObjectList*) statusBar() -> children();
  for ( uint i = 0; i < children -> count(); i ++ )
  {
    QObject* child = children -> at (i);
    if ( child -> inherits ("KStatusBarLabel") )
    {
      KStatusBarLabel* label = (KStatusBarLabel*) child;
      if ( label -> text() == i18n("Ready") )
        m_status_label = label;
      else if ( label -> text() == i18n("Idle") )
        m_state_label = label;
      else if ( label -> text().isEmpty() )
        m_progress_label = label;
    }
  }
}

void KPlayer::refreshSettings (void)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  int value = kPlayerProcess() -> volume();
  sliderAction ("player_volume") -> slider() -> setup (settings -> volumeMinimum(),
    settings -> volumeMaximum(), value, settings -> volumeMarks());
  popupAction ("popup_volume") -> slider() -> setup (settings -> volumeMinimum(),
    settings -> volumeMaximum(), value, settings -> volumeMarks());
  setVolume (value);
  value = kPlayerProcess() -> contrast();
  sliderAction ("video_contrast") -> slider() -> setup (settings -> contrastMinimum(),
    settings -> contrastMaximum(), value, settings -> contrastMarks());
  popupAction ("popup_contrast") -> slider() -> setup (settings -> contrastMinimum(),
    settings -> contrastMaximum(), value, settings -> contrastMarks());
  setContrast (value);
  value = kPlayerProcess() -> brightness();
  sliderAction ("video_brightness") -> slider() -> setup (settings -> brightnessMinimum(),
    settings -> brightnessMaximum(), value, settings -> brightnessMarks());
  popupAction ("popup_brightness") -> slider() -> setup (settings -> brightnessMinimum(),
    settings -> brightnessMaximum(), value, settings -> brightnessMarks());
  setBrightness (value);
  value = kPlayerProcess() -> hue();
  sliderAction ("video_hue") -> slider() -> setup (settings -> hueMinimum(),
    settings -> hueMaximum(), value, settings -> hueMarks());
  popupAction ("popup_hue") -> slider() -> setup (settings -> hueMinimum(),
    settings -> hueMaximum(), value, settings -> hueMarks());
  setHue (value);
  value = kPlayerProcess() -> saturation();
  sliderAction ("video_saturation") -> slider() -> setup (settings -> saturationMinimum(),
    settings -> saturationMaximum(), value, settings -> saturationMarks());
  popupAction ("popup_saturation") -> slider() -> setup (settings -> saturationMinimum(),
    settings -> saturationMaximum(), value, settings -> saturationMarks());
  setSaturation (value);
  if ( kPlayerProcess() -> length() > 0 )
    sliderAction ("player_progress") -> slider() ->
      setPageStep (int (roundf (kPlayerProcess() -> length() * settings -> progressMarks())));
  QApplication::postEvent (this, new QEvent (QEvent::LayoutHint));
//layout() -> activate();
  recentFilesAction() -> setMaxItems (settings -> recentFileListSize());
  if ( ! kPlayerProcess() -> url().isEmpty() )
  {
    if ( ! settings -> showFilePath() && ! kPlayerProcess() -> url().fileName().isEmpty() )
      setCaption (kPlayerProcess() -> url().fileName());
    else if ( kPlayerProcess() -> url().isLocalFile() && ! kPlayerProcess() -> url().path().isEmpty() )
      setCaption (kPlayerProcess() -> url().path());
    else
      setCaption (kPlayerProcess() -> url().prettyURL (0, KURL::StripFileProtocol));
  }
  m_updating = false;
}

void KPlayer::saveOptions (void)
{
  kdDebug() << "Saving options\n";
  KConfig* config = kapp -> config();
  saveMainWindowSettings (config, "General Options");
  Qt::Dock dock;
  int index = 0, offset = 0;
  bool newline = true;
  bool docked = getLocation (m_log, dock, index, newline, offset) &&
    (dock == Qt::DockTop || dock == DockBottom || dock == DockRight || dock == DockLeft);
  if ( docked )
  {
    config -> writeEntry ("Message Log Dock", dock);
    config -> writeEntry ("Message Log Index", index);
    config -> writeEntry ("Message Log New Line", newline);
    config -> writeEntry ("Message Log Offset", offset);
    kdDebug() << "Log dock " << dock << " " << index << " " << newline << " " << offset << " " << m_log -> x() << "x" << m_log -> y() << " " << m_log -> width() << "x" << m_log -> height() << "\n";
  }
  else
  {
    config -> deleteEntry ("Message Log Dock");
    config -> deleteEntry ("Message Log Index");
    config -> deleteEntry ("Message Log New Line");
    config -> deleteEntry ("Message Log Offset");
  }
  config -> writeEntry ("Message Log Left", m_log -> x());
  config -> writeEntry ("Message Log Top", m_log -> y());
  config -> writeEntry ("Message Log Width", m_log -> width());
  config -> writeEntry ("Message Log Height", m_log -> height());
  config -> writeEntry ("Main Window Docked", docked);
  if ( ! isFullScreen() )
  {
    config -> writeEntry ("Main Window Left", x());
    config -> writeEntry ("Main Window Top", y());
  }
  else
  {
    config -> writeEntry ("Main Window Left", m_normal_geometry.x());
    config -> writeEntry ("Main Window Top", m_normal_geometry.y());
  }
  config -> writeEntry ("Menu Bar Normally Visible", m_menubar_normally_visible);
  config -> writeEntry ("Menu Bar FullScreen Visible", m_menubar_fullscreen_visible);
  config -> writeEntry ("Status Bar Normally Visible", m_statusbar_normally_visible);
  config -> writeEntry ("Status Bar FullScreen Visible", m_statusbar_fullscreen_visible);
  config -> writeEntry ("Message Log Normally Visible", m_messagelog_normally_visible);
  config -> writeEntry ("Message Log FullScreen Visible", m_messagelog_fullscreen_visible);
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
  {
    config -> writeEntry (QString (m_toolbar[i].name) + " Normally Visible", m_toolbar[i].normally_visible);
    config -> writeEntry (QString (m_toolbar[i].name) + " FullScreen Visible", m_toolbar[i].fullscreen_visible);
  }
  recentFilesAction() -> saveEntries (config, "Recent Files");
  config -> setGroup ("Player Options");
  config -> writeEntry ("Maintain Aspect", kPlayerWidget() -> originalAspect());
  config -> writeEntry ("Frame Dropping", kPlayerProcess() -> frameDropping());
  config -> writeEntry ("Volume", kPlayerProcess() -> volume());
  config -> writeEntry ("Mute", toggleAction ("player_mute") -> isChecked());
  config -> writeEntry ("Contrast", kPlayerProcess() -> contrast());
  config -> writeEntry ("Brightness", kPlayerProcess() -> brightness());
  config -> writeEntry ("Hue", kPlayerProcess() -> hue());
  config -> writeEntry ("Saturation", kPlayerProcess() -> saturation());
  config -> writeEntry ("Full Screen", isFullScreen());
//config -> writeEntry ("Show Subtitles", kPlayerProcess() -> showSubtitles());
}

void KPlayer::readOptions (void)
{
  kdDebug() << "Reading options\n";
  KPlayerSettings* settings = kPlayerSettings();
  KConfig* config = kapp -> config();
  applyMainWindowSettings (config, "General Options");
  int x = config -> readNumEntry ("Main Window Left");
  int y = config -> readNumEntry ("Main Window Top");
  move (x, y);
  resize (450, 350);
  Qt::Dock dock = (Qt::Dock) config -> readNumEntry ("Message Log Dock", Qt::DockBottom);
  bool docked = config -> readBoolEntry ("Main Window Docked", true);
  bool newline = config -> readBoolEntry ("Message Log New Line", true);
  int index = config -> readNumEntry ("Message Log Index", 0);
  int offset = config -> readNumEntry ("Message Log Offset", 0);
  int left = config -> readNumEntry ("Message Log Left", 0);
  int top = config -> readNumEntry ("Message Log Top", 0);
  int width = config -> readNumEntry ("Message Log Width", 0);
  int height = config -> readNumEntry ("Message Log Height", 0);
  kdDebug() << "Log dock " << dock << " " << index << " " << newline << " " << offset << " " << left << "x" << top << " " << width << "x" << height << "\n";
  if ( width > 0 && height > 0 )
  {
    m_log -> setFixedExtentWidth (width);
    m_log -> setFixedExtentHeight (height);
  }
  moveDockWindow (m_log, dock, newline, index, offset);
  if ( ! docked )
  {
    m_log -> undock();
    if ( width > 0 && height > 0 )
      m_log -> setGeometry (left, top, width, height);
  }
  m_menubar_normally_visible = config -> readBoolEntry ("Menu Bar Normally Visible", m_menubar_normally_visible);
  m_menubar_fullscreen_visible = config -> readBoolEntry ("Menu Bar FullScreen Visible", m_menubar_fullscreen_visible);
  toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> setChecked (m_menubar_normally_visible);
  if ( m_menubar_normally_visible )
    menuBar() -> show();
  else
    menuBar() -> hide();
  m_statusbar_normally_visible = config -> readBoolEntry ("Status Bar Normally Visible", m_statusbar_normally_visible);
  m_statusbar_fullscreen_visible = config -> readBoolEntry ("Status Bar FullScreen Visible", m_statusbar_fullscreen_visible);
  toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> setChecked (m_statusbar_normally_visible);
  if ( m_statusbar_normally_visible )
    statusBar() -> show();
  else
    statusBar() -> hide();
  m_messagelog_normally_visible = config -> readBoolEntry ("Message Log Normally Visible", m_messagelog_normally_visible);
  m_messagelog_fullscreen_visible = config -> readBoolEntry ("Message Log FullScreen Visible", m_messagelog_fullscreen_visible);
  toggleAction ("options_show_log") -> setChecked (m_messagelog_normally_visible);
  if ( m_messagelog_normally_visible )
    m_log -> show();
  else
    m_log -> hide();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
  {
    m_toolbar[i].normally_visible = config -> readBoolEntry (QString (m_toolbar[i].name) + " Normally Visible", m_toolbar[i].normally_visible);
    m_toolbar[i].fullscreen_visible = config -> readBoolEntry (QString (m_toolbar[i].name) + " FullScreen Visible", m_toolbar[i].fullscreen_visible);
    toggleAction (m_toolbar[i].action) -> setChecked (m_toolbar[i].normally_visible);
    if ( m_toolbar[i].normally_visible )
      toolBar (m_toolbar[i].name) -> show();
    else
      toolBar (m_toolbar[i].name) -> hide();
  }
  KRecentFilesAction* recent = recentFilesAction();
  recent -> loadEntries (kapp -> config(), "Recent Files");
  recent -> setEnabled (recent -> items().count() > 0);
  config -> setGroup ("Player Options");
  kPlayerWidget() -> maintainOriginalAspect (config -> readBoolEntry ("Maintain Aspect", false));
  if ( kPlayerWidget() -> originalAspect() )
    toggleAction ("view_original_aspect") -> setChecked (true);
  int numEntry = config -> readNumEntry ("Frame Dropping", 0);
//kPlayerProcess() -> setFrameDropping ((KPlayerProcess::FrameDropping) numEntry);
  if ( numEntry == KPlayerProcess::Soft )
    toggleAction ("player_soft_frame_drop") -> activate();
  else if ( numEntry == KPlayerProcess::Hard )
    toggleAction ("player_hard_frame_drop") -> activate();
  if ( settings -> volumeReset() )
    setVolume (settings -> volumeDefault());
  else
  {
    setVolume (config -> readNumEntry ("Volume", settings -> volumeDefault()));
    if ( config -> readBoolEntry ("Mute", false) )
      toggleAction ("player_mute") -> activate();
  }
  if ( settings -> contrastReset() )
    setContrast (settings -> contrastDefault());
  else
    setContrast (config -> readNumEntry ("Contrast", settings -> contrastDefault()));
  if ( settings -> brightnessReset() )
    setBrightness (settings -> brightnessDefault());
  else
    setBrightness (config -> readNumEntry ("Brightness", settings -> brightnessDefault()));
  if ( settings -> hueReset() )
    setHue (settings -> hueDefault());
  else
    setHue (config -> readNumEntry ("Hue", settings -> hueDefault()));
  if ( settings -> saturationReset() )
    setSaturation (settings -> saturationDefault());
  else
    setSaturation (config -> readNumEntry ("Saturation", settings -> saturationDefault()));
  m_initial_fullscreen = config -> readBoolEntry ("Full Screen", false);
//kPlayerProcess() -> setShowSubtitles (config -> readBoolEntry ("Show Subtitles", true));
  refreshSettings();
}

void KPlayer::saveProperties (KConfig* config) const
{
  config -> writeEntry ("File URL", kPlayerProcess() -> url().url());
  config -> writeEntry ("Subtitle URL", kPlayerProcess() -> subtitleUrl().url());
}

void KPlayer::readProperties (KConfig* config)
{
  loadUrl (config -> readEntry ("File URL", ""), false);
  loadSubtitle (config -> readEntry ("Subtitle URL", ""));
}

void KPlayer::enableToolbar (int index)
{
  Toolbar& toolbar = m_toolbar [index];
  KToggleAction* action = toggleAction (toolbar.action);
//if ( action -> isEnabled() )
//  return;
  action -> setEnabled (true);
  if ( isFullScreen() ? toolbar.fullscreen_visible : toolbar.normally_visible )
  {
    action -> setChecked (true);
    toolBar (toolbar.name) -> show();
  }
}

void KPlayer::disableToolbar (int index)
{
  Toolbar& toolbar = m_toolbar [index];
  KToggleAction* action = toggleAction (toolbar.action);
//if ( ! action -> isEnabled() )
//  return;
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    toolBar (toolbar.name) -> hide();
  }
  action -> setEnabled (false);
}

void KPlayer::enablePlayerActions (void)
{
  KPlayerProcess::State state = kPlayerProcess() -> state();
  action ("player_play") -> setEnabled (! kPlayerProcess() -> url().isEmpty()
    && state != KPlayerProcess::Running && state != KPlayerProcess::Playing);
  action ("player_pause") -> setEnabled (state != KPlayerProcess::Idle);
  bool busy = state != KPlayerProcess::Idle;
  if ( ! busy )
  {
    QMouseEvent me (QEvent::MouseButtonRelease, QPoint(0,0), QPoint(0,0), QMouseEvent::LeftButton, QMouseEvent::NoButton);
    QApplication::sendEvent (sliderAction ("player_progress") -> slider(), &me);
  }
  sliderAction ("player_progress") -> slider() -> setEnabled (busy);
  action ("player_stop") -> setEnabled (busy);
  action ("player_forward") -> setEnabled (busy);
  action ("player_fast_forward") -> setEnabled (busy);
  action ("player_backward") -> setEnabled (busy);
  action ("player_fast_backward") -> setEnabled (busy);
  action ("player_start") -> setEnabled (busy);
  if ( kPlayerProcess() -> length() != 0 )
    enableToolbar (PROGRESS_TOOLBAR);
  else
    disableToolbar (PROGRESS_TOOLBAR);
  bool unpaused = state != KPlayerProcess::Paused;
  action ("player_volume_up") -> setEnabled (unpaused);
  action ("player_volume_down") -> setEnabled (unpaused);
  action ("player_mute") -> setEnabled (unpaused);
  action ("player_soft_frame_drop") -> setEnabled (unpaused);
  action ("player_hard_frame_drop") -> setEnabled (unpaused);
  action ("popup_volume") -> setEnabled (unpaused);
  sliderAction ("player_volume") -> slider() -> setEnabled (unpaused);
}

void KPlayer::enableVideoActions (void)
{
  bool video = ! kPlayerProcess() -> size().isEmpty();
  action ("view_full_screen") -> setEnabled (video || isFullScreen());
  action ("view_original_aspect") -> setEnabled (video);
  action ("view_current_aspect") -> setEnabled (video);
  action ("view_aspect_4_3") -> setEnabled (video);
  action ("view_aspect_16_9") -> setEnabled (video);
  bool unpaused = video && kPlayerProcess() -> state() != KPlayerProcess::Paused;
  action ("video_contrast_up") -> setEnabled (unpaused);
  action ("video_contrast_down") -> setEnabled (unpaused);
  action ("video_brightness_up") -> setEnabled (unpaused);
  action ("video_brightness_down") -> setEnabled (unpaused);
  action ("video_hue_up") -> setEnabled (unpaused);
  action ("video_hue_down") -> setEnabled (unpaused);
  action ("video_saturation_up") -> setEnabled (unpaused);
  action ("video_saturation_down") -> setEnabled (unpaused);
  action ("popup_contrast") -> setEnabled (unpaused);
  action ("popup_brightness") -> setEnabled (unpaused);
  action ("popup_hue") -> setEnabled (unpaused);
  action ("popup_saturation") -> setEnabled (unpaused);
  sliderAction ("video_contrast") -> slider() -> setEnabled (unpaused);
  sliderAction ("video_brightness") -> slider() -> setEnabled (unpaused);
  sliderAction ("video_hue") -> slider() -> setEnabled (unpaused);
  sliderAction ("video_saturation") -> slider() -> setEnabled (unpaused);
  for ( int i = FIRST_VIDEO_TOOLBAR; i < KPLAYER_TOOLBARS; i ++ )
    if ( video )
      enableToolbar (i);
    else
      disableToolbar (i);
  enableZoomActions();
  enableSubtitleActions();
  enableSubmenu (2, 2, unpaused);
}

void KPlayer::enableSubmenu (int index, int subindex, bool enable)
{
  QMenuData* data = menuBar();
  if ( ! data )
    return;
  int id = data -> idAt (index);
  if ( id == -1 )
    return;
  QMenuItem* item = data -> findItem (id);
  if ( ! item )
    return;
  data = item -> popup();
  if ( ! data )
    return;
  id = data -> idAt (data -> count() - subindex);
  if ( id == -1 )
    return;
  data -> setItemEnabled (id, enable);
}

void KPlayer::enableSubtitleActions (void)
{
  bool video = ! kPlayerProcess() -> size().isEmpty();
  bool subtitles = video && ! kPlayerProcess() -> subtitleUrl().isEmpty();
  bool playing = subtitles && kPlayerProcess() -> state() == KPlayerProcess::Playing;
  action ("subtitles_load") -> setEnabled (video);
  action ("subtitles_load_url") -> setEnabled (video);
  action ("subtitles_unload") -> setEnabled (subtitles);
  action ("subtitles_show") -> setEnabled (playing);
  action ("subtitles_move_down") -> setEnabled (playing);
  action ("subtitles_move_up") -> setEnabled (playing);
  action ("subtitles_delay_decrease") -> setEnabled (playing);
  action ("subtitles_delay_increase") -> setEnabled (playing);
  enableSubmenu (2, 1, playing);
}

void KPlayer::enableZoomActions (void)
{
  toggleAction ("view_zoom_1_2") -> setChecked (kPlayerWidget() -> isZoomFactor (1, 2));
  toggleAction ("view_zoom_1_1") -> setChecked (kPlayerWidget() -> isZoomFactor (1));
  toggleAction ("view_zoom_3_2") -> setChecked (kPlayerWidget() -> isZoomFactor (3, 2));
  toggleAction ("view_zoom_2_1") -> setChecked (kPlayerWidget() -> isZoomFactor (2));
  toggleAction ("view_zoom_5_2") -> setChecked (kPlayerWidget() -> isZoomFactor (5, 2));
  toggleAction ("view_zoom_3_1") -> setChecked (kPlayerWidget() -> isZoomFactor (3));
  bool canZoom = ! isFullScreen() && ! kPlayerProcess() -> size().isEmpty();
  toggleAction ("view_zoom_in") -> setEnabled (canZoom);
  toggleAction ("view_zoom_out") -> setEnabled (canZoom);
  toggleAction ("view_zoom_1_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_1_1") -> setEnabled (canZoom);
  toggleAction ("view_zoom_3_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_2_1") -> setEnabled (canZoom);
  toggleAction ("view_zoom_5_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_3_1") -> setEnabled (canZoom);
}

/*void KPlayer::checkBarActions (void)
{
  toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> setChecked (menuBar() -> isVisible());
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    toggleAction (m_toolbar[i].action) -> setChecked (toolBar (m_toolbar[i].name) -> isVisible());
  toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> setChecked (statusBar() -> isVisible());
}*/

void KPlayer::dragEnterEvent (QDragEnterEvent *event)
{
  event -> accept (KURLDrag::canDecode (event));
}

void KPlayer::dropEvent (QDropEvent* event)
{
  KURL::List urls;
  if ( ! KURLDrag::decode (event, urls) || urls.count() < 1 )
    return;
//for ( KURL::List::Iterator it = urls.begin(); it != urls.end(); ++ it )
  loadUrl (urls.first());
}

void KPlayer::start (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( isMinimized() )
  {
    kdDebug() << "Restoring minimized window\n";
    clearWState (WState_Visible); // or showNormal won't work
    showNormal();
  }
  else
    show();
  raise();
  setActiveWindow(); // doesn't work after restoring from minimized state
  kdDebug() << "Main window minimum size " << minimumWidth() << "x" << minimumHeight()
    << ", maximum size " << maximumWidth() << "x" << maximumHeight()
    << ", size " << width() << "x" << height() << "\n";
  KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
  if ( args -> count() > 0 )
    loadUrl (args -> url (args -> count() - 1));
  //loadUrl (KCmdLineArgs::makeURL (args -> arg (args -> count() - 1)));
  else if ( settings -> showOpenDialog() )
    fileOpen();
  args -> clear();
}

//bool KPlayerX11TestGrab (Display*, int);

/*bool KPlayer::event (QEvent* ev)
{
//kdDebug() << "KPlayer event: type " << ev -> type()
//  << " spontaneous " << ev -> spontaneous() << "\n";
//if ( ev -> type() == QEvent::LayoutHint && isFullScreen() )
//  return false;
  return KMainWindow::event (ev);
}*/

void KPlayer::contextMenuEvent (QContextMenuEvent* event)
{
  kdDebug() << "Main " << winId() << " wspace " << kPlayerWorkspace() -> winId()
    << " widget " << kPlayerWidget() -> winId() << "\n";
  QPopupMenu* popup = (QPopupMenu*) factory() -> container ("player_popup", this);
  popup -> popup (event -> globalPos());
  event -> accept();
}

void KPlayer::closeEvent (QCloseEvent* event)
{
//if ( isFullScreen() )
//  action ("view_full_screen") -> activate();
  saveOptions();
  enableScreenSaver();
  kPlayerProcess() -> kill();
  kPlayerWidget() -> close (true);
  KMainWindow::closeEvent (event);
}

void KPlayer::fileOpen (void)
{
  loadUrl (kPlayerWidget() -> openFile());
}

void KPlayer::fileOpenUrl (void)
{
  loadUrl (kPlayerWidget() -> openUrl());
}

void KPlayer::fileOpenRecent (const KURL& url)
{
  loadUrl (url);
  recentFilesAction() -> setCurrentItem (-1);
}

void KPlayer::fileOpenSubtitles (void)
{
  KURL url (kPlayerWidget() -> openSubtitle());
  if ( ! url.path().isEmpty() )
    loadSubtitle (url);
}

void KPlayer::fileOpenSubtitleUrl (void)
{
  KURL url (kPlayerWidget() -> openSubtitleUrl());
  if ( ! url.path().isEmpty() )
    loadSubtitle (url);
}

void KPlayer::fileUnloadSubtitles (void)
{
  loadSubtitle (KURL());
}

void KPlayer::fileClearLog (void)
{
  m_log -> clear();
}

void KPlayer::fileQuit (void)
{
  if ( m_state_label )
    m_state_label -> setText ("Closing");
  close();
}

void KPlayer::viewFullScreen (void)
{
  m_resizing = true;
  KToggleAction* action;
  if ( toggleAction ("view_full_screen") -> isChecked() )
  {
    m_normal_geometry.setRect (x(), y(), width(), height());
    kdDebug() << "Pre full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                          " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    kdDebug() << "                          " << m_normal_geometry.x() << "x" << m_normal_geometry.y() << " " << m_normal_geometry.width() << "x" << m_normal_geometry.height() << "\n";
    action = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar));
    action -> setChecked (m_menubar_fullscreen_visible);
    if ( m_menubar_fullscreen_visible && ! m_menubar_normally_visible )
      menuBar() -> show();
    else if ( ! m_menubar_fullscreen_visible && m_menubar_normally_visible )
      menuBar() -> hide();
    action = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar));
    action -> setChecked (m_statusbar_fullscreen_visible);
    if ( m_statusbar_fullscreen_visible && ! m_statusbar_normally_visible )
      statusBar() -> show();
    else if ( ! m_statusbar_fullscreen_visible && m_statusbar_normally_visible )
      statusBar() -> hide();
    action = toggleAction ("options_show_log");
    action -> setChecked (m_messagelog_fullscreen_visible);
    if ( m_messagelog_fullscreen_visible && ! m_messagelog_normally_visible )
      m_log -> show();
    else if ( ! m_messagelog_fullscreen_visible && m_messagelog_normally_visible )
      m_log -> hide();
    for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    {
      action = toggleAction (m_toolbar[i].action);
      action -> setChecked (m_toolbar[i].fullscreen_visible);
      if ( m_toolbar[i].fullscreen_visible && ! m_toolbar[i].normally_visible )
        toolBar (m_toolbar[i].name) -> show();
      else if ( ! m_toolbar[i].fullscreen_visible && m_toolbar[i].normally_visible )
        toolBar (m_toolbar[i].name) -> hide();
    }
    kPlayerWorkspace() -> setFreeResizeMode (false);
    m_resizing = false;
    showFullScreen();
    kdDebug() << "Full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                      " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    setGeometry (0, -1, width(), height() + 2);
    layout() -> activate();
    kdDebug() << "Full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                      " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
  }
  else
  {
    action = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar));
    action -> setChecked (m_menubar_normally_visible);
    if ( m_menubar_normally_visible && ! m_menubar_fullscreen_visible )
      menuBar() -> show();
    else if ( ! m_menubar_normally_visible && m_menubar_fullscreen_visible )
      menuBar() -> hide();
    action = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar));
    action -> setChecked (m_statusbar_normally_visible);
    if ( m_statusbar_normally_visible && ! m_statusbar_fullscreen_visible )
      statusBar() -> show();
    else if ( ! m_statusbar_normally_visible && m_statusbar_fullscreen_visible )
      statusBar() -> hide();
    action = toggleAction ("options_show_log");
    action -> setChecked (m_messagelog_normally_visible);
    if ( m_messagelog_normally_visible && ! m_messagelog_fullscreen_visible )
      m_log -> show();
    else if ( ! m_messagelog_normally_visible && m_messagelog_fullscreen_visible )
      m_log -> hide();
    for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    {
      action = toggleAction (m_toolbar[i].action);
      action -> setChecked (m_toolbar[i].normally_visible);
      if ( m_toolbar[i].normally_visible && ! m_toolbar[i].fullscreen_visible )
        toolBar (m_toolbar[i].name) -> show();
      else if ( ! m_toolbar[i].normally_visible && m_toolbar[i].fullscreen_visible )
        toolBar (m_toolbar[i].name) -> hide();
    }
    kPlayerWorkspace() -> setFreeResizeMode (true);
    kdDebug() << "Full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                      " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    setGeometry (0, 0, width(), height() - 2);
    kdDebug() << "Full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                      " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    m_resizing = false;
    setGeometry (m_normal_geometry);
    layout() -> activate();
    kdDebug() << "Full screen geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                      " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    showNormal();
    layout() -> activate();
    // geometry() != frameGeometry()
    // if ( width() != frameGeometry().width() || height() != frameGeometry().height() )
/*  if ( geometry() != m_normal_geometry || frameGeometry() != m_normal_geometry )
    {
      kdDebug() << "Normal geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
      kdDebug() << "                 " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
      kdDebug() << "                 " << m_normal_geometry.x() << "x" << m_normal_geometry.y() << " " << m_normal_geometry.width() << "x" << m_normal_geometry.height() << "\n";
//    setGeometry (m_normal_geometry.x() + 1, m_normal_geometry.y() + 1, m_normal_geometry.width() - 1, m_normal_geometry.height() - 1);
//    setGeometry (m_normal_geometry);
//    layout() -> activate();
    }*/
    kdDebug() << "Normal geometry: " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "                 " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
  }
}

void KPlayer::viewZoomIn (void)
{
  kPlayerWidget() -> zoomIn();
  zoom();
}

void KPlayer::viewZoomOut (void)
{
  kPlayerWidget() -> zoomOut();
  zoom();
}

void KPlayer::viewZoom12 (void)
{
  kPlayerWidget() -> setZoomFactor (1, 2);
  zoom();
}

void KPlayer::viewZoom11 (void)
{
  kPlayerWidget() -> setZoomFactor (1);
  zoom();
}

void KPlayer::viewZoom32 (void)
{
  kPlayerWidget() -> setZoomFactor (3, 2);
  zoom();
}

void KPlayer::viewZoom21 (void)
{
  kPlayerWidget() -> setZoomFactor (2);
  zoom();
}

void KPlayer::viewZoom52 (void)
{
  kPlayerWidget() -> setZoomFactor (5, 2);
  zoom();
}

void KPlayer::viewZoom31 (void)
{
  kPlayerWidget() -> setZoomFactor (3);
  zoom();
}

void KPlayer::viewMaintainOriginalAspect (void)
{
  kPlayerWidget() -> maintainOriginalAspect (toggleAction ("view_original_aspect") -> isChecked());
  zoom();
}

void KPlayer::viewMaintainCurrentAspect (void)
{
  kPlayerWidget() -> maintainCurrentAspect (toggleAction ("view_current_aspect") -> isChecked());
  zoom();
}

void KPlayer::viewAspect43 (void)
{
  if ( toggleAction ("view_aspect_4_3") -> isChecked() )
    kPlayerWidget() -> setFixedAspect (4, 3);
  else
    kPlayerWidget() -> setFixedAspect (0, 0);
  zoom();
}

void KPlayer::viewAspect169 (void)
{
  if ( toggleAction ("view_aspect_16_9") -> isChecked() )
    kPlayerWidget() -> setFixedAspect (16, 9);
  else
    kPlayerWidget() -> setFixedAspect (0, 0);
  zoom();
}

void KPlayer::loadSubtitle (KURL url)
{
  if ( url.path().isEmpty() )
    url = KURL();
  bool new_url = url != kPlayerProcess() -> subtitleUrl()
    && (! url.isEmpty() || ! kPlayerProcess() -> subtitleUrl().isEmpty());
  kdDebug() << "Subtitle '" << url.url() << "'\n";
  kdDebug() << "         '" << url.prettyURL (0, KURL::StripFileProtocol) << "'\n";
  if ( new_url )
    kPlayerProcess() -> setSubtitleUrl (url);
  KToggleAction* action = toggleAction ("subtitles_show");
  if ( ! action -> isChecked() )
    action -> activate();
  if ( ! new_url )
    return;
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
  {
    playerStop();
    playerProcessPlay();
  }
  enableSubtitleActions();
}

void KPlayer::loadUrl (KURL url, bool play)
{
  kdDebug() << "URL  '" << url.url() << "'\n";
  kdDebug() << "     '" << url.prettyURL (0, KURL::StripFileProtocol) << "'\n";
  if ( url.path().isEmpty() && url.host().isEmpty() )
    return;
  KPlayerSettings* settings = kPlayerSettings();
  play = play && settings -> startPlayingImmediately();
  if ( url == kPlayerProcess() -> url() )
  {
    if ( play )
      playerPlay();
    return;
  }
  kPlayerProcess() -> kill();
  kPlayerProcess() -> setUrl (url);
  KToggleAction* action = toggleAction ("view_current_aspect");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> maintainCurrentAspect (false);
  }
  action = toggleAction ("view_aspect_4_3");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> setFixedAspect (0, 0);
  }
  action = toggleAction ("view_aspect_16_9");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> setFixedAspect (0, 0);
  }
  if ( m_progress_label )
    m_progress_label -> setText ("");
  if ( ! settings -> showFilePath() && ! kPlayerProcess() -> url().fileName().isEmpty() )
    setCaption (kPlayerProcess() -> url().fileName());
  else if ( kPlayerProcess() -> url().isLocalFile() && ! kPlayerProcess() -> url().path().isEmpty() )
    setCaption (kPlayerProcess() -> url().path());
  else
    setCaption (kPlayerProcess() -> url().prettyURL (0, KURL::StripFileProtocol));
/*if ( settings -> showFilePath() )
    setCaption (kPlayerProcess() -> url().prettyURL (0, KURL::StripFileProtocol));
  else
    setCaption (kPlayerProcess() -> url().fileName());*/
  if ( settings -> volumeReset() && settings -> volumeEvery() == 0 )
  {
    setVolume (settings -> volumeDefault());
    action = toggleAction ("player_mute");
    if ( action -> isChecked() )
      action -> activate();
  }
  if ( settings -> contrastReset() && settings -> contrastEvery() == 0 )
    setContrast (settings -> contrastDefault());
  if ( settings -> brightnessReset() && settings -> brightnessEvery() == 0 )
    setBrightness (settings -> brightnessDefault());
  if ( settings -> hueReset() && settings -> hueEvery() == 0 )
    setHue (settings -> hueDefault());
  if ( settings -> saturationReset() && settings -> saturationEvery() == 0 )
    setSaturation (settings -> saturationDefault());
  action = toggleAction ("subtitles_show");
  if ( ! action -> isChecked() && ! kPlayerProcess() -> subtitleUrl().isEmpty() )
    action -> activate();
  if ( ! kPlayerProcess() -> get_info() )
  {
    m_log -> addLine (i18n("Could not start mplayer"));
    playerErrorDetected();
  }
  else if ( play )
    playerProcessPlay();
  if ( settings -> recentFileListSize() < 1 )
    return;
  KRecentFilesAction* recent = recentFilesAction();
  recent -> setEnabled (true);
  recent -> addURL (kPlayerProcess() -> url());
}

void KPlayer::playerPlay (void)
{
  if ( kPlayerProcess() -> state() == KPlayerProcess::Paused )
    kPlayerProcess() -> pause();
  else if ( kPlayerProcess() -> state() == KPlayerProcess::Idle )
    playerProcessPlay();
}

void KPlayer::playerProcessPlay (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( settings -> clearMessagesBeforePlaying() )
    m_log -> clear();
  KToggleAction* toggle = toggleAction ("options_show_log");
  if ( settings -> showMessagesBeforePlaying() && ! toggle -> isChecked() )
    toggle -> activate();
  if ( ! kPlayerProcess() -> play() )
  {
    m_log -> addLine (i18n("Could not start mplayer"));
    playerErrorDetected();
  }
}

void KPlayer::playerPause (void)
{
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    kPlayerProcess() -> pause();
  else
  {
    KToggleAction* action = toggleAction ("player_pause");
    if ( action -> isChecked() )
      action -> setChecked (false);
  }
}

void KPlayer::playerStop (void)
{
  kPlayerProcess() -> stop();
}

void KPlayer::playerForward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressNormalSeekUnits() ? settings -> progressNormalSeek() :
    kPlayerProcess() -> length() == 0 ? 10 :
    int (roundf (kPlayerProcess() -> length() * settings -> progressNormalSeek() / 100));
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayer::playerFastForward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressFastSeekUnits() ? settings -> progressFastSeek() :
    kPlayerProcess() -> length() == 0 ? 60 :
    int (roundf (kPlayerProcess() -> length() * settings -> progressFastSeek() / 100));
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayer::playerBackward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressNormalSeekUnits() ? settings -> progressNormalSeek() :
    kPlayerProcess() -> length() == 0 ? 10 :
    int (roundf (kPlayerProcess() -> length() * settings -> progressNormalSeek() / 100));
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayer::playerFastBackward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressFastSeekUnits() ? settings -> progressFastSeek() :
    kPlayerProcess() -> length() == 0 ? 60 :
    int (roundf (kPlayerProcess() -> length() * settings -> progressFastSeek() / 100));
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayer::playerStart (void)
{
  kPlayerProcess() -> absoluteSeek (0);
}

void KPlayer::setVolume (int volume)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  volume = limit (volume, settings -> volumeMinimum(), settings -> volumeMaximum());
  if ( volume != kPlayerProcess() -> volume() )
    kPlayerProcess() -> setVolume (volume);
  KPlayerSlider* slider = sliderAction ("player_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  slider = popupAction ("popup_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  m_updating = false;
}

void KPlayer::setContrast (int contrast)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  contrast = limit (contrast, settings -> contrastMinimum(), settings -> contrastMaximum());
  if ( contrast != kPlayerProcess() -> contrast() )
    kPlayerProcess() -> setContrast (contrast);
  KPlayerSlider* slider = sliderAction ("video_contrast") -> slider();
  if ( contrast != slider -> value() )
    slider -> setValue (contrast);
  slider = popupAction ("popup_contrast") -> slider();
  if ( contrast != slider -> value() )
    slider -> setValue (contrast);
  m_updating = false;
}

void KPlayer::setBrightness (int brightness)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  brightness = limit (brightness, settings -> brightnessMinimum(), settings -> brightnessMaximum());
  if ( brightness != kPlayerProcess() -> brightness() )
    kPlayerProcess() -> setBrightness (brightness);
  KPlayerSlider* slider = sliderAction ("video_brightness") -> slider();
  if ( brightness != slider -> value() )
    slider -> setValue (brightness);
  slider = popupAction ("popup_brightness") -> slider();
  if ( brightness != slider -> value() )
    slider -> setValue (brightness);
  m_updating = false;
}

void KPlayer::setHue (int hue)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  hue = limit (hue, settings -> hueMinimum(), settings -> hueMaximum());
  if ( hue != kPlayerProcess() -> hue() )
    kPlayerProcess() -> setHue (hue);
  KPlayerSlider* slider = sliderAction ("video_hue") -> slider();
  if ( hue != slider -> value() )
    slider -> setValue (hue);
  slider = popupAction ("popup_hue") -> slider();
  if ( hue != slider -> value() )
    slider -> setValue (hue);
  m_updating = false;
}

void KPlayer::setSaturation (int saturation)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  saturation = limit (saturation, settings -> saturationMinimum(), settings -> saturationMaximum());
  if ( saturation != kPlayerProcess() -> saturation() )
    kPlayerProcess() -> setSaturation (saturation);
  KPlayerSlider* slider = sliderAction ("video_saturation") -> slider();
  if ( saturation != slider -> value() )
    slider -> setValue (saturation);
  slider = popupAction ("popup_saturation") -> slider();
  if ( saturation != slider -> value() )
    slider -> setValue (saturation);
  m_updating = false;
}

void KPlayer::playerVolumeUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> volume() < settings -> volumeMaximum() )
    setVolume (kPlayerProcess() -> volume() + settings -> volumeStep());
}

void KPlayer::playerVolumeDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> volume() > settings -> volumeMinimum() )
    setVolume (kPlayerProcess() -> volume() - settings -> volumeStep());
}

void KPlayer::playerMute (void)
{
  kPlayerProcess() -> mute();
/*if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    kPlayerProcess() -> mute();
  else
  {
    KToggleAction* action = toggleAction ("player_mute");
    if ( action -> isChecked() )
      action -> setChecked (false);
  }*/
}

void KPlayer::playerSoftFrameDrop (void)
{
  kPlayerProcess() -> setFrameDropping (toggleAction ("player_soft_frame_drop") -> isChecked() ?
    KPlayerProcess::Soft : KPlayerProcess::None);
}

void KPlayer::playerHardFrameDrop (void)
{
  kPlayerProcess() -> setFrameDropping (toggleAction ("player_hard_frame_drop") -> isChecked() ?
    KPlayerProcess::Hard : KPlayerProcess::None);
}

void KPlayer::playerSubtitlesShow (void)
{
  kPlayerProcess() -> setShowSubtitles (toggleAction ("subtitles_show") -> isChecked());
}

void KPlayer::playerSubtitlesMoveDown (void)
{
  kPlayerProcess() -> subtitleMove (-1);
}

void KPlayer::playerSubtitlesMoveUp (void)
{
  kPlayerProcess() -> subtitleMove (1);
}

void KPlayer::playerSubtitlesDelayDecrease (void)
{
  kPlayerProcess() -> subtitleDelay (-1);
}

void KPlayer::playerSubtitlesDelayIncrease (void)
{
  kPlayerProcess() -> subtitleDelay (1);
}

void KPlayer::videoContrastUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> contrast() < settings -> contrastMaximum() )
    setContrast (kPlayerProcess() -> contrast() + settings -> contrastStep());
}

void KPlayer::videoContrastDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> contrast() > settings -> contrastMinimum() )
    setContrast (kPlayerProcess() -> contrast() - settings -> contrastStep());
}

void KPlayer::videoBrightnessUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> brightness() < settings -> brightnessMaximum() )
    setBrightness (kPlayerProcess() -> brightness() + settings -> brightnessStep());
}

void KPlayer::videoBrightnessDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> brightness() > settings -> brightnessMinimum() )
    setBrightness (kPlayerProcess() -> brightness() - settings -> brightnessStep());
}

void KPlayer::videoHueUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> hue() < settings -> hueMaximum() )
    setHue (kPlayerProcess() -> hue() + settings -> hueStep());
}

void KPlayer::videoHueDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> hue() > settings -> hueMinimum() )
    setHue (kPlayerProcess() -> hue() - settings -> hueStep());
}

void KPlayer::videoSaturationUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> saturation() < settings -> saturationMaximum() )
    setSaturation (kPlayerProcess() -> saturation() + settings -> saturationStep());
}

void KPlayer::videoSaturationDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> saturation() > settings -> saturationMinimum() )
    setSaturation (kPlayerProcess() -> saturation() - settings -> saturationStep());
}

void KPlayer::viewMenubar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> isChecked();
  if ( isFullScreen() )
    m_menubar_fullscreen_visible = visible;
  else
    m_menubar_normally_visible = visible;
  if ( visible )
    menuBar() -> show();
  else
    menuBar() -> hide();
}

void KPlayer::viewStatusbar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> isChecked();
  if ( isFullScreen() )
    m_statusbar_fullscreen_visible = visible;
  else
    m_statusbar_normally_visible = visible;
  if ( visible )
    statusBar() -> show();
  else
    statusBar() -> hide();
}

void KPlayer::viewMessageLog (void)
{
  bool visible = toggleAction ("options_show_log") -> isChecked();
  if ( isFullScreen() )
    m_messagelog_fullscreen_visible = visible;
  else
    m_messagelog_normally_visible = visible;
  if ( visible )
    m_log -> show();
  else
    m_log -> hide();
//widgetResized();
}

void KPlayer::viewMainToolbar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowToolbar)) -> isChecked();
  if ( isFullScreen() )
    m_toolbar[MAIN_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[MAIN_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("mainToolBar") -> show();
  else
    toolBar ("mainToolBar") -> hide();
}

void KPlayer::viewProgressToolbar (void)
{
  bool visible = toggleAction ("settings_progress_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[PROGRESS_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[PROGRESS_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("progressToolBar") -> show();
  else
    toolBar ("progressToolBar") -> hide();
}

void KPlayer::viewVolumeToolbar (void)
{
  bool visible = toggleAction ("settings_volume_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[VOLUME_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[VOLUME_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("volumeToolBar") -> show();
  else
    toolBar ("volumeToolBar") -> hide();
}

void KPlayer::viewContrastToolbar (void)
{
  bool visible = toggleAction ("settings_contrast_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[CONTRAST_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[CONTRAST_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("contrastToolBar") -> show();
  else
    toolBar ("contrastToolBar") -> hide();
}

void KPlayer::viewBrightnessToolbar (void)
{
  bool visible = toggleAction ("settings_brightness_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[BRIGHTNESS_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[BRIGHTNESS_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("brightnessToolBar") -> show();
  else
    toolBar ("brightnessToolBar") -> hide();
}

void KPlayer::viewHueToolbar (void)
{
  bool visible = toggleAction ("settings_hue_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[HUE_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[HUE_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("hueToolBar") -> show();
  else
    toolBar ("hueToolBar") -> hide();
}

void KPlayer::viewSaturationToolbar (void)
{
  bool visible = toggleAction ("settings_saturation_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[SATURATION_TOOLBAR].fullscreen_visible = visible;
  else
    m_toolbar[SATURATION_TOOLBAR].normally_visible = visible;
  if ( visible )
    toolBar ("saturationToolBar") -> show();
  else
    toolBar ("saturationToolBar") -> hide();
}

void KPlayer::settingsKeyBindings (void)
{
//KKeyDialog::configure(); - reportedly was buggy in KDE 3.0x
  KKeyDialog keydlg (true);
  keydlg.insert (actionCollection());
  keydlg.configure (true);
}

void KPlayer::settingsConfigure (void)
{
  KPlayerSettingsDialog().exec();
}

void KPlayer::progressChanged (int progress)
{
  if ( m_updating )
    return;
  kdDebug() << "Seek to " << progress << ": " << kPlayerProcess() -> position() << " => " << ((progress + 50) / 100) << " / " << kPlayerProcess() -> length() << "\n";
  kPlayerProcess() -> absoluteSeek ((progress + 50) / 100);
/*int seek = int (roundf (float (progress) / 100 - kPlayerProcess() -> position()));
  if ( seek == 0 )
    return;
  kPlayerProcess() -> relativeSeek (seek);*/
}

void KPlayer::volumeChanged (int volume)
{
  if ( ! m_updating )
    setVolume (volume);
}

void KPlayer::contrastChanged (int contrast)
{
  if ( ! m_updating )
    setContrast (contrast);
}

void KPlayer::brightnessChanged (int brightness)
{
  if ( ! m_updating )
    setBrightness (brightness);
}

void KPlayer::hueChanged (int hue)
{
  if ( ! m_updating )
    setHue (hue);
}

void KPlayer::saturationChanged (int saturation)
{
  if ( ! m_updating )
    setSaturation (saturation);
}

void KPlayer::playerStateChanged (KPlayerProcess::State state)
{
  static KPlayerProcess::State previous = KPlayerProcess::Idle;
  static const char* stateMessages [4] = { "Idle", "Running", "Playing", "Paused" };
  kdDebug() << "State change received: " << state << "\n";
  KPlayerSettings* settings = kPlayerSettings();
  KToggleAction* toggle = toggleAction ("player_pause");
  if ( state != KPlayerProcess::Paused && toggle -> isChecked() )
    toggle -> setChecked (false);
/*toggle = toggleAction ("player_mute");
  if ( state == KPlayerProcess::Idle && toggle -> isChecked() )
    toggle -> setChecked (false);*/
//statusBar() -> changeItem (stateMessages [state], ID_STATE_MSG);
  if ( m_state_label )
    m_state_label -> setText (stateMessages [state]);
  if ( state == KPlayerProcess::Idle )
    playerProgressChanged (0);
  if ( m_initial_fullscreen && state == KPlayerProcess::Playing )
  {
    if ( kPlayerProcess() -> size().isEmpty() )
      m_initial_fullscreen = false;
    else
      playerSizeAvailable();
  }
  toggle = toggleAction ("options_show_log");
  if ( state == KPlayerProcess::Idle && previous != state
      && settings -> showMessagesAfterPlaying() && ! toggle -> isChecked() )
    toggle -> activate();
  enablePlayerActions();
  enableVideoActions();
  if ( state == KPlayerProcess::Playing )
    disableScreenSaver();
  else
    enableScreenSaver();
  previous = state;
}

void KPlayer::disableScreenSaver (void)
{
  if ( ! kPlayerSettings() -> disableScreenSaver() || kPlayerProcess() -> size().isEmpty() )
    return;
  QByteArray data, reply;
  QCString type;
  if ( ! kapp -> dcopClient() -> call ("kdesktop", "KScreensaverIface", "isEnabled()", data, type, reply)
      || type != "bool" )
  {
    kdWarning() << "SS: Could not get screen saver status\n";
    return;
  }
  QDataStream replyStream (reply, IO_ReadOnly);
  bool enabled = false;
  replyStream >> enabled;
  if ( ! enabled )
  {
    kdDebug() << "SS: Screen saver is not enabled\n";
    return;
  }
  QDataStream argStream (data, IO_WriteOnly);
  argStream << false;
  if ( kapp -> dcopClient() -> send ("kdesktop", "KScreensaverIface", "enable(bool)", data) )
    kdDebug() << "SS: Screensaver successfully disabled\n";
  else
    kdWarning() << "SS: Could not disable screensaver\n";
  m_enable_screen_saver = true;
}

void KPlayer::enableScreenSaver (void)
{
  if ( ! m_enable_screen_saver )
    return;
  QByteArray data;
  QDataStream argStream (data, IO_WriteOnly);
  argStream << true;
  if ( kapp -> dcopClient() -> send ("kdesktop", "KScreensaverIface", "enable(bool)", data) )
    kdDebug() << "SS: Screensaver successfully enabled\n";
  else
    kdWarning() << "SS: Could not enable screensaver\n";
  m_enable_screen_saver = false;
}

void KPlayer::playerProgressChanged (float progress)
{
  static float prev_progress = -1;
  float diff = progress - prev_progress;
  prev_progress = progress;
  if ( diff <= 0 && diff > -0.15 && progress > 0 )
    return;
  m_updating = true;
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  int maxValue = slider -> maxValue();
  if ( maxValue )
  {
    int value = int (roundf (progress * 100));
    if ( value > maxValue )
      slider -> setMaxValue (value);
    slider -> setValue (value);
  }
  float length = kPlayerProcess() -> length();
  int progressHour = int (progress) / 3600, lengthHour = int (length) / 3600;
  int progressMinute = (int (progress) - progressHour * 3600) / 60;
  int lengthMinute = (int (length) - lengthHour * 3600) / 60;
  if ( progressHour > 0 || progressMinute > 0 )
    progress -= progressHour * 3600 + progressMinute * 60;
  if ( lengthHour > 0 || lengthMinute > 0 )
    length -= lengthHour * 3600 + lengthMinute * 60;
  QString s;
  if ( length > 0 )
  {
    if ( progressHour > 0 || lengthHour > 0 )
      s.sprintf ("%u:%02u:%04.1f / %u:%02u:%04.1f", progressHour, progressMinute, progress, lengthHour, lengthMinute, length);
    else if ( progressMinute > 0 || lengthMinute > 0 )
      s.sprintf ("%u:%04.1f / %u:%04.1f", progressMinute, progress, lengthMinute, length);
    else
      s.sprintf ("%03.1f / %03.1f", progress, length);
  }
  else
  {
    if ( progressHour > 0 )
      s.sprintf ("%u:%02u:%04.1f", progressHour, progressMinute, progress);
    else if ( progressMinute > 0 )
      s.sprintf ("%u:%04.1f", progressMinute, progress);
    else
      s.sprintf ("%03.1f", progress);
  }
//statusBar() -> changeItem (s, ID_PROGRESS_MSG);
  if ( m_progress_label )
    m_progress_label -> setText (s);
  m_updating = false;
}

void KPlayer::playerInfoAvailable (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  kdDebug() << "Info available. Detected length: " << kPlayerProcess() -> length() << "\n";
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  slider -> setMaxValue (int (roundf (kPlayerProcess() -> length() * 100)));
  slider -> setPageStep (int (roundf (kPlayerProcess() -> length() * settings -> progressMarks())));
  enablePlayerActions();
}

void KPlayer::playerSizeAvailable (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  kdDebug() << "Size Available. Video size " << kPlayerProcess() -> width() << "x" << kPlayerProcess() -> height() << "\n";
  KAction* fsaction = action ("view_full_screen");
  if ( kPlayerProcess() -> size().isEmpty() )
  {
    if ( isFullScreen() )
      fsaction -> activate();
    m_initial_fullscreen = false;
    int i;
    for ( i = 0; i < KPLAYER_TOOLBARS; i ++ )
      if ( toolBar (m_toolbar[i].name) -> orientation() == Qt::Vertical
          && toggleAction (m_toolbar[i].action) -> isChecked() )
        break;
    if ( i == KPLAYER_TOOLBARS )
    {
      kPlayerWidget() -> setVideoSize (QSize (m_log -> isHidden() || m_video_size.isEmpty() ?
        settings -> minimumInitialWidth() : m_video_size.width(), 0));
      zoom();
      QApplication::postEvent (this, new QEvent (QEvent::LayoutHint));
    }
/*  for ( uint i = 0; i < cl -> count(); i ++ )
      if ( cl -> at (i) -> inherits ("QWidget") )
        kdDebug() << "*** Child " << cl -> at (i) -> className() << " min height hint " <<
          (((QWidget*) cl -> at (i)) -> minimumSizeHint().height()) << "\n"; */
  }
  else
  {
    if ( ! isFullScreen() )
    {
      int d = 1, n = (settings -> minimumInitialWidth() - 1) / kPlayerProcess() -> width();
      if ( n > 0 && kPlayerProcess() -> width() * n + kPlayerProcess() -> width() / 2 >= settings -> minimumInitialWidth() )
        n *= d = 2;
      kdDebug() << "Initial Zoom Factor " << (n + 1) << " / " << d << "\n";
      kPlayerWidget() -> setZoomFactor (n + 1, d);
      zoom();
    }
    if ( m_initial_fullscreen && kPlayerProcess() -> state() != KPlayerProcess::Idle )
    {
      kdDebug() << "Initial full screen\n";
      if ( ! isFullScreen() )
        fsaction -> activate();
      m_initial_fullscreen = false;
    }
  }
  enableVideoActions();
}

void KPlayer::playerMessageReceived (const char* message)
{
  if ( ! *message )
    return;
  log (message);
}

void KPlayer::playerErrorDetected (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  KToggleAction* toggle = toggleAction ("options_show_log");
  kdDebug() << "Error detected: " << settings -> showMessagesOnError() << " " << toggle -> isChecked() << "\n";
  if ( settings -> showMessagesOnError() && ! toggle -> isChecked() )
    toggle -> activate();
}

void KPlayer::logWindowHidden (void)
{
  kdDebug() << "Log window hidden signal\n";
  toggleAction ("options_show_log") -> setChecked (false);
  if ( isFullScreen() )
    m_messagelog_fullscreen_visible = false;
  else
    m_messagelog_normally_visible = false;
}

void KPlayer::log (QString message)
{
  if ( message.isEmpty() )
    return;
  m_log -> addLine (message);
}

void KPlayer::setStatusText (const QString& text)
{
  statusBar() -> clear();
//statusBar() -> changeItem (text, ID_STATUS_MSG);
  if ( m_status_label )
    m_status_label -> setText (text);
}

void KPlayer::setStatusMessage (const QString& text)
{
  if ( text.isEmpty() )
    return;
  statusBar() -> message (text);
  m_progress_label -> hide();
  m_state_label -> hide();
}

void KPlayer::clearStatusMessage (void)
{
  statusBar() -> clear();
  m_state_label -> show();
  m_progress_label -> show();
}

void KPlayer::resizeEvent (QResizeEvent* event)
{
  kdDebug() << "Window " << event -> oldSize(). width() << "x" << event -> oldSize(). height()
    << " => " << event -> size(). width() << "x" << event -> size(). height() << "\n";
  KMainWindow::resizeEvent (event);
}

void KPlayer::widgetResized (void)
{
  if ( m_resizing )
    return;
  kdDebug() << "Widget resized event\n";
  if ( ! isFullScreen() )
  {
    //Qt::Dock dock;
    //int index, offset, size;
    //bool newline;
    //if ( getLocation (m_log, dock, index, newline, offset) )
    //  size = dock == Qt::DockLeft || dock == Qt::DockRight ? m_log -> width() : m_log -> height();
    //else
    //  dock = Qt::DockTornOff;
    //    && dock != Qt::DockTornOff && dock == m_log_dock && size == m_log_size )
    if ( m_previous_size == size() && ! m_video_size.isEmpty() )
      kPlayerWidget() -> setVideoSize (m_video_size);
    else // if ( ! centralWidget() -> size().isEmpty() )
      kPlayerWidget() -> setVideoSize (centralWidget() -> size(), true, m_previous_size.width() == width());
  }
  zoom();
}

void KPlayer::zoom (void)
{
  if ( m_resizing )
    return;
  m_resizing = true;
  if ( isFullScreen() || isMaximized() || frameGeometry() == QApplication::desktop() -> geometry() )
  {
    kPlayerWorkspace() -> setVideoSize();
    kdDebug() << "Full screen zoom done\n";
    enableZoomActions();
    m_resizing = false;
    return;
  }
  //QApplication::sendPostedEvents (this, QEvent::Resize);
  //QApplication::sendPostedEvents (this, QEvent::LayoutHint);
  //QApplication::sendPostedEvents (centralWidget(), QEvent::Resize);
  //QApplication::sendPostedEvents (kPlayerWidget(), QEvent::Resize);
  layout() -> activate();
  QSize video (kPlayerWidget() -> videoSize()), cwsize (centralWidget() -> size());
  QSize target ((size() + video - cwsize).expandedTo (sizeHint()));
  m_video_size = video;
  kdDebug() << "Window " << geometry().x() << "x" << geometry().y() << " " << width() << "x" << height()
    << "\nWFrame " << x() << "x" << y() << " " << frameGeometry().width() << "x" << frameGeometry().height()
    << "\nWSpace " << cwsize.width() << "x" << cwsize.height()
    << "\nWidget " << kPlayerWidget() -> size().width() << "x" << kPlayerWidget() -> size().height()
    << "\nVdSize " << video.width() << "x" << video.height()
    << "\nTarget " << target.width() << "x" << target.height() << "\n";
  kdDebug() << " Total  " << minimumSize().width() << "x" << minimumSize().height()
    << " " << minimumSizeHint().width() << "x" << minimumSizeHint().height()
    << " " << sizeHint().width() << "x" << sizeHint().height()
    << "\n Top    " << topDock() -> minimumSize().width() << "x" << topDock() -> minimumSize().height()
    << " " << topDock() -> minimumSizeHint().width() << "x" << topDock() -> minimumSizeHint().height()
    << " " << topDock() -> sizeHint().width() << "x" << topDock() -> sizeHint().height()
    << "\n Bottom " << bottomDock() -> minimumSize().width() << "x" << bottomDock() -> minimumSize().height()
    << " " << bottomDock() -> minimumSizeHint().width() << "x" << bottomDock() -> minimumSizeHint().height()
    << " " << bottomDock() -> sizeHint().width() << "x" << bottomDock() -> sizeHint().height()
    << "\n Layout " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
    << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height()
    << "\n        " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
    << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height() << "\n";
  int i = 0;
  while ( target != size() && i ++ < 5 ) // cwsize != video
  {
    resize (target);
    layout() -> activate();
    //QApplication::sendPostedEvents (this, QEvent::Resize);
    //QApplication::sendPostedEvents (this, QEvent::LayoutHint);
    //QApplication::sendPostedEvents (centralWidget(), QEvent::Resize);
    //QApplication::sendPostedEvents (kPlayerWidget(), QEvent::Resize);
    cwsize = centralWidget() -> size();
    target = (size() + video - cwsize).expandedTo (sizeHint());
    kdDebug() << "  Window " << width() << "x" << height()
      << "\n  WSpace " << cwsize.width() << "x" << cwsize.height()
      << "\n  Widget " << kPlayerWidget() -> size().width() << "x" << kPlayerWidget() -> size().height()
      << "\n  Target " << target.width() << "x" << target.height() << "\n";
    kdDebug() << " Total  " << minimumSize().width() << "x" << minimumSize().height()
      << " " << minimumSizeHint().width() << "x" << minimumSizeHint().height()
      << " " << sizeHint().width() << "x" << sizeHint().height()
      << "\n Top    " << topDock() -> minimumSize().width() << "x" << topDock() -> minimumSize().height()
      << " " << topDock() -> minimumSizeHint().width() << "x" << topDock() -> minimumSizeHint().height()
      << " " << topDock() -> sizeHint().width() << "x" << topDock() -> sizeHint().height()
      << "\n Bottom " << bottomDock() -> minimumSize().width() << "x" << bottomDock() -> minimumSize().height()
      << " " << bottomDock() -> minimumSizeHint().width() << "x" << bottomDock() -> minimumSizeHint().height()
      << " " << bottomDock() -> sizeHint().width() << "x" << bottomDock() -> sizeHint().height()
      << "\n Layout " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height()
      << "\n        " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height() << "\n";
  }
  kdDebug() << "Frame " << x() << "x" << y() << " " << frameGeometry().width() << "x" << frameGeometry().height()
    << "\nDesktop " << QApplication::desktop() -> width() << "x" << QApplication::desktop() -> height() << "\n";
  Qt::Dock dock;
  int index, offset;
  bool newline;
  if ( (frameGeometry().width() > QApplication::desktop() -> width() ||
    frameGeometry().height() > QApplication::desktop() -> height())
    && ! m_log -> isHidden() && getLocation (m_log, dock, index, newline, offset) )
  {
    kdDebug() << "Log dock " << dock << " " << index << " " << newline << " " << offset << " " << m_log -> x() << "x" << m_log -> y() << " " << m_log -> width() << "x" << m_log -> height() << "\n";
    kdDebug() << "Minimum log size " << m_log -> minimumWidth() << "x" << m_log -> minimumHeight() << "\n";
    if ( (dock == Qt::DockLeft || dock == Qt::DockRight)
      && frameGeometry().width() > QApplication::desktop() -> width()
      && m_log -> width() - m_log -> minimumWidth() >= frameGeometry().width() - QApplication::desktop() -> width() )
    {
      kdDebug() << "New log width " << (m_log -> width() - frameGeometry().width() + QApplication::desktop() -> width()) << "\n";
      m_log -> setFixedExtentWidth (m_log -> width() - frameGeometry().width() + QApplication::desktop() -> width());
      layout() -> activate();
    }
    else if ( (dock == Qt::DockTop || dock == Qt::DockBottom)
      && frameGeometry().height() > QApplication::desktop() -> height()
      && m_log -> height() - m_log -> minimumHeight() >= frameGeometry().height() - QApplication::desktop() -> height() )
    {
      kdDebug() << "New log height " << (m_log -> height() - frameGeometry().height() + QApplication::desktop() -> height()) << "\n";
      m_log -> setFixedExtentHeight (m_log -> height() - frameGeometry().height() + QApplication::desktop() -> height());
      layout() -> activate();
    }
    else
    {
      m_log -> undock();
      layout() -> activate();
    }
    cwsize = centralWidget() -> size();
    target = (size() + video - cwsize).expandedTo (sizeHint());
    kdDebug() << "Frame " << x() << "x" << y() << " " << frameGeometry().width() << "x" << frameGeometry().height()
      << "\nDesktop " << QApplication::desktop() -> width() << "x" << QApplication::desktop() -> height() << "\n";
    kdDebug() << "Window " << x() << "x" << y() << " " << width() << "x" << height()
      << "\nWSpace " << cwsize.width() << "x" << cwsize.height()
      << "\nWidget " << kPlayerWidget() -> size().width() << "x" << kPlayerWidget() -> size().height()
      << "\nVdSize " << video.width() << "x" << video.height()
      << "\nTarget " << target.width() << "x" << target.height() << "\n";
    kdDebug() << " Total  " << minimumSize().width() << "x" << minimumSize().height()
      << " " << minimumSizeHint().width() << "x" << minimumSizeHint().height()
      << " " << sizeHint().width() << "x" << sizeHint().height()
      << "\n Top    " << topDock() -> minimumSize().width() << "x" << topDock() -> minimumSize().height()
      << " " << topDock() -> minimumSizeHint().width() << "x" << topDock() -> minimumSizeHint().height()
      << " " << topDock() -> sizeHint().width() << "x" << topDock() -> sizeHint().height()
      << "\n Bottom " << bottomDock() -> minimumSize().width() << "x" << bottomDock() -> minimumSize().height()
      << " " << bottomDock() -> minimumSizeHint().width() << "x" << bottomDock() -> minimumSizeHint().height()
      << " " << bottomDock() -> sizeHint().width() << "x" << bottomDock() -> sizeHint().height()
      << "\n Layout " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height()
      << "\n        " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height() << "\n";
  }
  i = 0;
  while ( target != size() && i ++ < 5 ) // cwsize != video
  {
    resize (target);
    layout() -> activate();
    cwsize = centralWidget() -> size();
    target = (size() + video - cwsize).expandedTo (sizeHint());
    kdDebug() << "  Window " << width() << "x" << height()
      << "\n  WSpace " << cwsize.width() << "x" << cwsize.height()
      << "\n  Widget " << kPlayerWidget() -> size().width() << "x" << kPlayerWidget() -> size().height()
      << "\n  Target " << target.width() << "x" << target.height() << "\n";
    kdDebug() << " Total  " << minimumSize().width() << "x" << minimumSize().height()
      << " " << minimumSizeHint().width() << "x" << minimumSizeHint().height()
      << " " << sizeHint().width() << "x" << sizeHint().height()
      << "\n Top    " << topDock() -> minimumSize().width() << "x" << topDock() -> minimumSize().height()
      << " " << topDock() -> minimumSizeHint().width() << "x" << topDock() -> minimumSizeHint().height()
      << " " << topDock() -> sizeHint().width() << "x" << topDock() -> sizeHint().height()
      << "\n Bottom " << bottomDock() -> minimumSize().width() << "x" << bottomDock() -> minimumSize().height()
      << " " << bottomDock() -> minimumSizeHint().width() << "x" << bottomDock() -> minimumSizeHint().height()
      << " " << bottomDock() -> sizeHint().width() << "x" << bottomDock() -> sizeHint().height()
      << "\n Layout " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height()
      << "\n        " << layout() -> minimumSize().width() << "x" << layout() -> minimumSize().height()
      << " " << layout() -> sizeHint().width() << "x" << layout() -> sizeHint().height() << "\n";
  }
  kdDebug() << "Frame " << x() << "x" << y() << " " << frameGeometry().width() << "x" << frameGeometry().height()
    << "\nDesktop " << QApplication::desktop() -> width() << "x" << QApplication::desktop() -> height() << "\n";
  int xc = x() + frameGeometry().width() - QApplication::desktop() -> width();
  int yc = y() + frameGeometry().height() - QApplication::desktop() -> height();
  if ( xc > 0 || yc > 0 )
  {
    if ( xc <= 0 )
      xc = x();
    else if ( QApplication::desktop() -> width() > frameGeometry().width() )
      xc = QApplication::desktop() -> width() - frameGeometry().width();
    else
      xc = 0;
    if ( yc <= 0 )
      yc = y();
    else if ( QApplication::desktop() -> height() > frameGeometry().height() )
      yc = QApplication::desktop() -> height() - frameGeometry().height();
    else
      yc = 0;
    kdDebug() << "Moving to " << xc << "x" << yc << "\n";
    move (xc, yc);
  }
  m_previous_size = size();
//layout() -> invalidate();
  QApplication::postEvent (this, new QEvent (QEvent::LayoutHint));
  enableZoomActions();
//checkBarActions();
  m_resizing = false;
  kdDebug() << "Normal zoom done\n";
}

/*void KPlayer::barOrientationChanged (Orientation)
{
  kdDebug() << "A bar orientation has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}

void KPlayer::barPlaceChanged (QDockWindow::Place)
{
  kdDebug() << "A bar place has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}

void KPlayer::barVisibilityChanged (bool)
{
  kdDebug() << "A bar visibility has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}*/
